//-----------------------------------------------------------------------------
//  (c) 2008 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
  \file
  Test the implementation of CFloat
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#   include <Windows.h>
#   include <commctrl.h>
#endif

#include "AliasTest.h"
#include "GenApi/GenApi.h"
#include <limits>


using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( AliasTestSuite, "GenApiTest");



void AliasTestSuite::TestBasics()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring CameraDescription =
    CDFHeader +
    "                                         \n"
    "<Float Name=\"Abs\">                     \n"
    "    <pCastAlias>Raw</pCastAlias>                 \n"
    "    <Value>42.0</Value>                 \n"
    "</Float>                                 \n"
    "                                         \n"
    "<Integer Name=\"Raw\">                   \n"
    "    <pCastAlias>Abs</pCastAlias>                 \n"
    "    <Value>42</Value>                    \n"
    "</Integer>                               \n"
    "                                         \n"
    "<Integer Name=\"NoAlias\">               \n"
    "    <Value>0815</Value>                  \n"
    "</Integer>                               \n"

    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CFloatPtr ptrAbs = Camera._GetNode( "Abs" );
    CPPUNIT_ASSERT( (bool) ptrAbs );

    CIntegerPtr ptrRaw = Camera._GetNode( "Raw" );
    CPPUNIT_ASSERT( (bool) ptrRaw );

    CIntegerPtr ptrNoAlias = Camera._GetNode( "NoAlias" );
    CPPUNIT_ASSERT( (bool) ptrNoAlias );

    // retrieving the alias manually
    {
        CNodePtr ptrAlias = ptrAbs->GetNode()->GetCastAlias();
        CPPUNIT_ASSERT( (bool) ptrAlias );
        CPPUNIT_ASSERT_EQUAL( gcstring("Raw"), ptrAlias->GetName() );

        CIntegerPtr ptrIntAlias(ptrAlias);
        CPPUNIT_ASSERT( (bool) ptrIntAlias );
        CPPUNIT_ASSERT_EQUAL( ptrRaw->GetValue(), ptrIntAlias->GetValue() );
    }
    // retrieving the alias via helper function
    {
        CPPUNIT_ASSERT_EQUAL( ptrRaw->GetValue(), ptrAbs.GetIntAlias()->GetValue() );
    }
    // the same with references
    CIntegerRef refRaw;
    refRaw.SetReference( ptrRaw );
    CFloatRef refAbs;
    CPPUNIT_ASSERT_THROW_EX (refAbs.GetEnumAlias(), AccessException);
    refAbs.SetReference( ptrAbs );
    CIntegerRef refNoAlias;
    refNoAlias.SetReference( ptrNoAlias );
    {
        CPPUNIT_ASSERT_EQUAL( refRaw.GetValue(), refAbs.GetIntAlias()->GetValue() );
        CPPUNIT_ASSERT_EQUAL( (IEnumeration*)NULL, refAbs.GetEnumAlias() );
    }
}

void AliasTestSuite::TestWithSwissKnife()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring CameraDescription =
    CDFHeader +
    "                                              \n"
    "<Converter Name=\"Abs\">                      \n"
    "    <ToolTip>20 us per tick</ToolTip>         \n"
    "    <pCastAlias>Raw</pCastAlias>              \n"
    "    <FormulaTo> FROM / 0.00002</FormulaTo>    \n"
    "    <FormulaFrom> TO * 0.00002</FormulaFrom>  \n"
    "    <pValue>Raw</pValue>                      \n"
    "    <Unit>s</Unit>                            \n"
    "    <Slope>Increasing</Slope>                 \n"
    "</Converter>                                  \n"
    "                                             \n"
    "<Integer Name=\"Raw\">                       \n"
    "    <pCastAlias>Abs</pCastAlias>             \n"
    "    <Value>10</Value>                        \n"
    "</Integer>                                   \n"
    "                                             \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CFloatPtr ptrAbs = Camera._GetNode( "Abs" );
    CPPUNIT_ASSERT( (bool) ptrAbs );

    CIntegerPtr ptrRaw = Camera._GetNode( "Raw" );
    CPPUNIT_ASSERT( (bool) ptrRaw );

    *ptrAbs = 0.01; // 10 ms
    CPPUNIT_ASSERT_EQUAL( (int64_t) 500LL, ptrAbs.GetIntAlias()->GetValue() );

}


void AliasTestSuite::TestSliderUseCases()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    // This example shows how to get rid of Abs / Raw features using an Alias node.

    // For demonstration a digital shift Gain is used, i.e.
    //     GainAbs = 1 << GainRaw = 2^GainRaw
    //     GainRaw = ld(GainAbs) = ln(GainAbs)/ln(2)
    //
    // which yields the following possible values:
    //     GainRaw, GainAbs
    //        0        1.0
    //        1        2.0
    //        2        4.0
    //        3        8.0
    //        4       16.0

    // "GainRaw" and "GainAbs" are implemented using a Converter node
    // However they could also be implemented by using two camera registers
    // The important thing is that changing one othe the two values must automatically
    // change the other, too.

    // The "Gain" feature is in essence the same as the "GainAbs" feature. However since
    // for backward compatibility "GainAbs" and "GainRaw" must be available "Gain" is made a separate node

    // Note however that "GainAbs" and "GainRaw" are made invisible so that they will not show
    // up in a GUI but will be present in an API to be used by legacy software.

    // The <pAlias> entry of the "Gain" node shows that "GainRaw" is it's alias.
    // The same entry is (optionally) present in the "GainAbs" node.

    gcstring CameraDescription =
    CDFHeader +
    "                                              \n"
    "<Category Name=\"Root\">                      \n"
    "    <pFeature>Gain</pFeature>                 \n"
    "    <pFeature>GainAbs</pFeature>              \n"
    "    <pFeature>GainRaw</pFeature>              \n"
    "</Category>                                   \n"
    "                                              \n"
    "<Float Name=\"Gain\">                         \n"
    "    <Visibility>Beginner</Visibility>         \n"
    "    <pCastAlias>GainRaw</pCastAlias>          \n"
    "    <pValue>GainAbs</pValue>                  \n"
    "</Float>                                      \n"
    "                                              \n"
    "<Converter Name=\"GainAbs\">                  \n"
    "    <Visibility>Invisible</Visibility>        \n"
    "    <pCastAlias>GainRaw</pCastAlias>          \n"
    "    <FormulaTo> LN(FROM)/LN(2) </FormulaTo>   \n"
    "    <FormulaFrom><![CDATA[ 1<<TO ]]></FormulaFrom>        \n"
    "    <pValue>GainRaw</pValue>                  \n"
    "    <Slope>Increasing</Slope>                 \n"
    "</Converter>                                  \n"
    "                                              \n"
    "<Integer Name=\"GainRaw\">                    \n"
    "    <Visibility>Invisible</Visibility>        \n"
    "    <Value>0</Value>                          \n"
    "    <Min>0</Min>                              \n"
    "    <Max>4</Max>                              \n"
    "</Integer>                                    \n"
    "                                              \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // Get the "Gain" node
    CFloatPtr ptrGain = Camera._GetNode( "Gain" );
    CPPUNIT_ASSERT( (bool) ptrGain );

    // Check if an alias is present
    // Note that .GetXXXAlias() is accessed via the . (dot) operator, not via the -> (arrow) operator
    // because GetXXXAlias is a method of the CFloatPtr smart pointer not the IFloat interface
    if( ptrGain.GetIntAlias() )
        ; // Show a slider with an edit box
    else
        ; // Show an edit box only

    // the following assumes an alias is present
    CPPUNIT_ASSERT( NULL != ptrGain.GetIntAlias() );

    // compute the number of ticks to draw on the slider
    int NumberOfTicks = (int)(( ptrGain.GetIntAlias()->GetMax() - ptrGain.GetIntAlias()->GetMin() ) / ptrGain.GetIntAlias()->GetInc());
    CPPUNIT_ASSERT_EQUAL( 4, NumberOfTicks );

    // initialize the gain
    *ptrGain = 1.0;

    // fill the edit box
    gcstring EditBox = ptrGain->ToString();
    CPPUNIT_ASSERT_EQUAL( gcstring("1"), EditBox );

    // user sets a gain not fitting exactly Gain's granularity
    // the gain will jump backwards to the nearest possible value
    ptrGain->FromString("5.2");
    CPPUNIT_ASSERT_EQUAL( gcstring("4"), ptrGain->ToString());
    // or will forward back to the nearest possible value
    ptrGain->FromString("5.7");
    CPPUNIT_ASSERT_EQUAL( gcstring("8"), ptrGain->ToString());

    // Tick the slider up
    int64_t TickUp = min( ptrGain.GetIntAlias()->GetValue() + ptrGain.GetIntAlias()->GetInc(), ptrGain.GetIntAlias()->GetMax());
    ptrGain.GetIntAlias()->SetValue(TickUp);
    CPPUNIT_ASSERT_EQUAL( gcstring("16"), ptrGain->ToString());

    // Try again (but now the maximum is already reached)
    TickUp = min( ptrGain.GetIntAlias()->GetValue() + ptrGain.GetIntAlias()->GetInc(), ptrGain.GetIntAlias()->GetMax());
    ptrGain.GetIntAlias()->SetValue(TickUp);
    CPPUNIT_ASSERT_EQUAL( gcstring("16"), ptrGain->ToString());

}


void AliasTestSuite::TestJoeCustomer()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring CameraDescription =
        CDFHeader +

        "<Float Name=\"GainA\">                   \n"
        "    <pAlias>GainAbs</pAlias>             \n"
        "    <pValue>GainAbs</pValue>             \n"
        "</Float>                                 \n"
        "                                         \n"
        "<Float Name=\"GainR1\">                   \n"
        "    <pAlias>GainRaw1</pAlias>             \n"
        "    <pValue>GainRaw1</pValue>             \n"
        "</Float>                                 \n"
        "                                         \n"
        // the following converter emulates a digital shift Gain
        //     Gain = 2^GainRaw = 1 << GainRaw
        //     GainRaw = ld(Abs) = ln(Abs)/ln(2)
        "<Converter Name=\"GainR2\">                        \n"
        "    <pAlias>GainRaw2</pAlias>                       \n"
        "    <FormulaTo> LN(FROM)/LN(2) </FormulaTo>        \n"
        "    <FormulaFrom><![CDATA[ 1<<TO ]]></FormulaFrom> \n"
        "    <pValue>GainRaw2</pValue>                       \n"
        "    <Slope>Increasing</Slope>                      \n"
        "</Converter>                                       \n"
        "                                         \n"
        "<Float Name=\"GainL\">                   \n"
        "    <pAlias>GainList</pAlias>            \n"
        "    <pValue>GainList</pValue>            \n"
        "</Float>                                 \n"
        "                                         \n"
        "                                         \n"
        "<Float Name=\"GainAbs\">                 \n"
        "    <Value>1.0</Value>                   \n"
        "</Float>                                 \n"
        "                                         \n"
        "<Integer Name=\"GainRaw1\">               \n"
        "    <Value>1</Value>                     \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Integer Name=\"GainRaw2\">               \n"
        "    <Value>0</Value>                     \n"
        "    <Min>0</Min>                         \n"
        "    <Max>20</Max>                       \n"
        "    <Inc>2</Inc>                         \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Enumeration Name=\"GainList\">             \n"
        "    <EnumEntry Name=\"Low\">                \n"
        "        <Value>0</Value>                    \n"
        "        <NumericValue>1.0</NumericValue>    \n"
        "   </EnumEntry>                             \n"
        "    <EnumEntry Name=\"Medium\">             \n"
        "        <Value>1</Value>                    \n"
        "        <NumericValue>50.0</NumericValue>   \n"
        "   </EnumEntry>                             \n"
        "    <EnumEntry Name=\"Heigh\">              \n"
        "        <Value>2</Value>                    \n"
        "        <NumericValue>100.0</NumericValue>  \n"
        "   </EnumEntry>                             \n"
        "    <Value>1</Value>                        \n"
        "</Enumeration>                              \n"

        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CFloatPtr ptrGainA = Camera._GetNode( "GainA" );
    CFloatPtr ptrGainR1 = Camera._GetNode( "GainR1" );
    CFloatPtr ptrGainR2 = Camera._GetNode( "GainR2" );
    CFloatPtr ptrGainL = Camera._GetNode( "GainL" );

    *ptrGainA = 41.6;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 41.6, *ptrGainA(), std::numeric_limits<double>::epsilon() );

    *ptrGainR1 = 41.6;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 42.0, *ptrGainR1(), std::numeric_limits<double>::epsilon() );
    *ptrGainR1 = 17.1;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 17.0, *ptrGainR1(), std::numeric_limits<double>::epsilon() );

    *ptrGainR2 = 5.2;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 4.0, *ptrGainR2(), std::numeric_limits<double>::epsilon() );
    *ptrGainR2 = 15.1;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 16.0, *ptrGainR2(), std::numeric_limits<double>::epsilon() );

    *ptrGainL = 41.6;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 50.0, *ptrGainL(), std::numeric_limits<double>::epsilon() );
    *ptrGainL = 76.0;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 100.0, *ptrGainL(), std::numeric_limits<double>::epsilon() );

}

void AliasTestSuite::TestVeronicaVendor()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring CameraDescription =
        CDFHeader +

        "<Float Name=\"GainA\">                   \n"
        "    <pCastAlias>GainAbs</pCastAlias>     \n"
        "    <pValue>GainAbs</pValue>             \n"
        "</Float>                                 \n"
        "                                         \n"

        // the following converter emulates a digital shift Gain
        //     Gain = 2^GainRaw = 1 << GainRaw
        //     GainRaw = ld(Abs) = ln(Abs)/ln(2)
        "<Converter Name=\"GainR\">                         \n"
        "    <pCastAlias>GainRaw</pCastAlias>               \n"
        "    <FormulaTo> LN(FROM)/LN(2) </FormulaTo>        \n"
        "    <FormulaFrom><![CDATA[ 1<<TO ]]></FormulaFrom> \n"
        "    <pValue>GainRaw</pValue>                       \n"
        "    <Slope>Increasing</Slope>                      \n"
        "</Converter>                                       \n"
        "                                         \n"
        "<Float Name=\"GainL\">                   \n"
        "    <pCastAlias>GainList</pCastAlias>    \n"
        "    <pValue>GainList</pValue>            \n"
        "</Float>                                 \n"
        "                                         \n"
        "<Float Name=\"GainAbs\">                 \n"
        "    <Value>1.0</Value>                   \n"
        "</Float>                                 \n"
        "                                         \n"
        "<Integer Name=\"GainRaw\">               \n"
        "    <Value>0</Value>                     \n"
        "    <Min>0</Min>                         \n"
        "    <Max>20</Max>                       \n"
        "    <Inc>2</Inc>                         \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Enumeration Name=\"GainList\">             \n"
        "    <EnumEntry Name=\"Low\">                \n"
        "        <Value>0</Value>                    \n"
        "        <NumericValue>1.0</NumericValue>    \n"
        "   </EnumEntry>                             \n"
        "    <EnumEntry Name=\"Medium\">             \n"
        "        <Value>1</Value>                    \n"
        "        <NumericValue>50.0</NumericValue>   \n"
        "   </EnumEntry>                             \n"
        "    <EnumEntry Name=\"High\">               \n"
        "        <Value>2</Value>                    \n"
        "        <NumericValue>100.0</NumericValue>  \n"
        "   </EnumEntry>                             \n"
        "    <Value>1</Value>                        \n"
        "</Enumeration>                              \n"

        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CFloatPtr ptrGainA = Camera._GetNode( "GainA" );
    CFloatPtr ptrGainR = Camera._GetNode( "GainR" );
    CFloatPtr ptrGainL = Camera._GetNode( "GainL" );

    RunVeronicaTest(ptrGainA);
    RunVeronicaTest(ptrGainR);
    RunVeronicaTest(ptrGainL);
}

void AliasTestSuite::RunVeronicaTest( CFloatPtr &ptrGain )
{

    if( IInteger *pGainRaw = ptrGain.GetIntAlias() )
    {
        cout << "==> Gain with IInteger alias" << endl;

        cout << "GainMin = " << ptrGain->GetMin() << endl;

        cout << "GainMax = " << ptrGain->GetMax() << endl;

        int64_t NumTics = (pGainRaw->GetMax() - pGainRaw->GetMin()) / pGainRaw->GetInc();
        cout << "NumTics = " << NumTics << endl;

        pGainRaw->SetValue( pGainRaw->GetMin() + (int64_t)((double)pGainRaw->GetInc() * (double)NumTics * 0.1) );
        cout << pGainRaw->GetValue() << endl;
        cout << "10% slider range = " << ptrGain->GetValue() << endl;

        pGainRaw->SetValue( pGainRaw->GetValue() + pGainRaw->GetInc() );
        cout << "+ 1 Tic = " << ptrGain->GetValue() << endl;
    }
    else if( IEnumeration *pGainList = ptrGain.GetEnumAlias() )
    {
        cout << "==> Gain with IEnumeration alias" << endl;

        NodeList_t EnumEntries;
        pGainList ->GetEntries(EnumEntries);
        for( NodeList_t::iterator ptr=EnumEntries.begin(); ptr!=EnumEntries.end(); ptr++ )
        {
            CEnumEntryPtr ptrEnumEntry(*ptr);
            cout << ptrEnumEntry->GetSymbolic() << ", IntValue = " << ptrEnumEntry->GetValue() << ", NumericValue = " << ptrEnumEntry->GetNumericValue() << endl;
        }

        pGainList->FromString("High");
        cout << "High Gain = " << ptrGain->GetValue() << endl;
    }
    else
    {
        cout << "==> Gain with no alias" << endl;

        cout << "Gain = " << ptrGain->GetValue() << endl;
    }
}


