//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------

/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "ImposeTest.h"
#include "TestPort.h"
#include <limits>
#include "GenApi/GenApi.h"
#include "GenApi/Test/PropertyBag.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;
using namespace Test_PropertyBag;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ImposeTestSuite, "GenApiTest");

void ImposeTestSuite::TestImposeAccessMode()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                        \n"
    "    <Node Name=\"Node\">                                \n"
    "    </Node>                                            \n"
    "                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePtr ptrNode = Camera._GetNode("Node");
    CPPUNIT_ASSERT( ptrNode.IsValid() );

    CPPUNIT_ASSERT_EQUAL( RW, ptrNode->GetAccessMode() );

    ptrNode->ImposeAccessMode(RO);
    CPPUNIT_ASSERT_EQUAL( RO, ptrNode->GetAccessMode() );

    ptrNode->ImposeAccessMode(WO);
    CPPUNIT_ASSERT_EQUAL( WO, ptrNode->GetAccessMode() );

    ptrNode->ImposeAccessMode(NA);
    CPPUNIT_ASSERT_EQUAL( NA, ptrNode->GetAccessMode() );

    ptrNode->ImposeAccessMode(NI);
    CPPUNIT_ASSERT_EQUAL( NI, ptrNode->GetAccessMode() );

}

void ImposeTestSuite::TestImposeAccessModeElement()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<Integer Name=\"Value\">                        \n"
    "    <ImposedAccessMode>RO</ImposedAccessMode>    \n"
    "    <pValue>ValueReg</pValue>                    \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<IntReg Name=\"ValueReg\">                        \n"
    "        <Address>0x0104</Address>                \n"
    "        <Length>4</Length>                        \n"
    "        <AccessMode>RW</AccessMode>                \n"
     "        <pPort>Port</pPort>                        \n"
    "        <Sign>Unsigned</Sign>                    \n"
     "        <Endianess>LittleEndian</Endianess>        \n"
    "        <Representation>Linear</Representation>    \n"
    "</IntReg>                                        \n"
    "                                                \n"
    "<Port Name=\"Port\"/>                            \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CTestPort Port;
    Port.CreateEntry(0x0104, (uint32_t)1024, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CIntegerPtr ptrValue = Camera._GetNode("Value");
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    CPPUNIT_ASSERT_EQUAL( RO, ptrValue->GetAccessMode() );


}



void ImposeTestSuite::TestImposeVisibility()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                        \n"
    "    <Integer Name=\"Node\">                                \n"
    "        <Value>0</Value>                                \n"
    "    </Integer>                                            \n"
    "                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePtr ptrNode = Camera._GetNode("Node");
    CPPUNIT_ASSERT( ptrNode.IsValid() );

    CPPUNIT_ASSERT_EQUAL( Beginner, ptrNode->GetVisibility() );

    ptrNode->ImposeVisibility(Expert);
    CPPUNIT_ASSERT_EQUAL( Expert, ptrNode->GetVisibility() );

    ptrNode->ImposeVisibility(Guru);
    CPPUNIT_ASSERT_EQUAL( Guru, ptrNode->GetVisibility() );

    ptrNode->ImposeVisibility(Invisible);
    CPPUNIT_ASSERT_EQUAL( Invisible, ptrNode->GetVisibility() );
}

void ImposeTestSuite::TestImposeMinMax()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                        \n"
    "    <Integer Name=\"ValueInt\">                         \n"
    "       <Value>0</Value>                                 \n"
    "    </Integer>                                          \n"
    "                                                        \n"
    "    <Float Name=\"ValueFloat\">                         \n"
    "       <Value>0.0</Value>                               \n"
    "    </Float>                                            \n"
    "                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrInteger = Camera._GetNode("ValueInt");
    CPPUNIT_ASSERT( ptrInteger.IsValid() );

    CPPUNIT_ASSERT_EQUAL( INT64_MIN, ptrInteger->GetMin() );
    CPPUNIT_ASSERT_EQUAL( INT64_MAX, ptrInteger->GetMax() );

    ptrInteger->ImposeMin( 42 );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, ptrInteger->GetMin() );

    ptrInteger->ImposeMax( -13 );
    CPPUNIT_ASSERT_EQUAL( (int64_t) -13LL, ptrInteger->GetMax() );

    CFloatPtr ptrFloat = Camera._GetNode("ValueFloat");
    CPPUNIT_ASSERT( ptrFloat.IsValid() );

    CPPUNIT_ASSERT_DOUBLES_EQUAL( -(std::numeric_limits<double>::max)(), ptrFloat->GetMin(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( (std::numeric_limits<double>::max)(), ptrFloat->GetMax(), std::numeric_limits<double>::epsilon() );

    ptrFloat->ImposeMin( 42.0 );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 42.0, ptrFloat->GetMin(), std::numeric_limits<double>::epsilon() );

    ptrFloat->ImposeMax( -13.0 );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( -13.0, ptrFloat->GetMax(), std::numeric_limits<double>::epsilon() );

}

void ImposeTestSuite::TestPropertyBag()
{
    CPropertyBag PropertyBag;
    PropertyBag._LoadXMLFromFile("Test", "PropertyBag");

    CPPUNIT_ASSERT_EQUAL( (int64_t) 0LL, PropertyBag.GainRaw.GetMin() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 100LL, PropertyBag.GainRaw.GetMax() );

    PropertyBag.GainRaw.ImposeMin(-1);
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0LL, PropertyBag.GainRaw.GetMin() );
    PropertyBag.GainRaw.ImposeMin(1);
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1LL, PropertyBag.GainRaw.GetMin() );

    PropertyBag.GainRaw.ImposeMax(101);
    CPPUNIT_ASSERT_EQUAL( (int64_t) 100LL, PropertyBag.GainRaw.GetMax() );
    PropertyBag.GainRaw.ImposeMax(99);
    CPPUNIT_ASSERT_EQUAL( (int64_t) 99LL, PropertyBag.GainRaw.GetMax() );



    CPPUNIT_ASSERT_DOUBLES_EQUAL_EX( 0.0, PropertyBag.GainAbs.GetMin() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL_EX( 100.0, PropertyBag.GainAbs.GetMax() );

    PropertyBag.GainAbs.ImposeMin(-1.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL_EX( 0.0, PropertyBag.GainAbs.GetMin() );
    PropertyBag.GainAbs.ImposeMin(1.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL_EX( 1.0, PropertyBag.GainAbs.GetMin() );

    PropertyBag.GainAbs.ImposeMax(101.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL_EX( 100.0, PropertyBag.GainAbs.GetMax() );
    PropertyBag.GainAbs.ImposeMax(99.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL_EX( 99.0, PropertyBag.GainAbs.GetMax() );
}

