//-----------------------------------------------------------------------------
//  (c) 2010 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
  \file
  Test the implementation of NodeMap persistence
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "NodeMapPersistenceTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"


using namespace std;
using namespace GenICam;
using namespace GenApi;


CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( NodeMapPersistenceTestSuite, "GenApiTest");

void NodeMapPersistenceTestSuite::TestSimple()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestSimple.xml|gxml

    <Category Name="Root">
        <pFeature>IntegerValue</pFeature>
        <pFeature>IntegerValueNotStreamable</pFeature>
        <pFeature>FloatValue</pFeature>
        <pFeature>EnumerationValue</pFeature>
    </Category>

    <Integer Name="IntegerValue">
        <Streamable>Yes</Streamable>
        <Value>17</Value>
    </Integer>

    <Integer Name="IntegerValueNotStreamable">
        <Streamable>No</Streamable>
        <Value>17</Value>
    </Integer>

    <Float Name="FloatValue">
        <Streamable>Yes</Streamable>
        <Value>17.0</Value>
    </Float>

    <Enumeration Name="EnumerationValue">
        <Streamable>Yes</Streamable>
        <EnumEntry Name="EnumValue1">
            <Value>10</Value>
        </EnumEntry>
        <EnumEntry Name="EnumValue2">
            <Value>20</Value>
        </EnumEntry>
        <pValue>EnumInt</pValue>
    </Enumeration>

    <Integer Name="EnumInt">
        <Value>10</Value>
    </Integer>

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestSimple");

    NODE_POINTER( Integer, IntegerValue )
    NODE_POINTER( Integer, IntegerValueNotStreamable )
    NODE_POINTER( Float, FloatValue )
    NODE_POINTER( Enumeration, EnumerationValue )

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)3, NumBagEntries);

    // verify the == operator
        CFeatureBag FeatureBagCopy;
        FeatureBagCopy.StoreToBag(Camera._Ptr);
        CPPUNIT_ASSERT (FeatureBag == FeatureBagCopy);
        CFeatureBag FeatureBagEmpty;
        CPPUNIT_ASSERT (!(FeatureBag == FeatureBagEmpty));

    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    // print the string's content to stdout (don't forget to reset the filte pointer to the begin)
    cout << File.rdbuf();
    File.seekg( ios::beg );

    // load string into another(!) bag
    CFeatureBag FeatureBag2;
    File >> FeatureBag2;

    // change the values
    *ptrIntegerValue = 13;
    *ptrIntegerValueNotStreamable = 13;
    *ptrFloatValue = 13.0;
    *ptrEnumerationValue = "EnumValue2";

    // load the values from FeatureBag and verify
    FeatureBag2.LoadFromBag(Camera._Ptr);
    CPPUNIT_ASSERT_EQUAL( (int64_t)17, ptrIntegerValue->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)13, ptrIntegerValueNotStreamable->GetValue() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 17.0, ptrFloatValue->GetValue(), 0.001 );
    CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue1"), ptrEnumerationValue->ToString() );

}

void NodeMapPersistenceTestSuite::TestSelector()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestSelector.xml|gxml

    <Category Name="Root">
        <pFeature>Entry</pFeature>
        <pFeature>Selector</pFeature>
    </Category>

    <IntReg Name="Entry">
        <Streamable>Yes</Streamable>
        <pAddress>Selector</pAddress>
        <Length>4</Length>
        <AccessMode>RW</AccessMode>
        <pPort>MyPort</pPort>
        <Sign>Signed</Sign>
        <Endianess>LittleEndian</Endianess>
    </IntReg>

    <Integer Name="Selector">
        <!-- The selector does not need (and indeed should not) be streamable -->
        <Streamable>No</Streamable>
        <Value>48</Value>
        <Min>40</Min>
        <Max>49</Max>
        <pSelected>Entry</pSelected>
    </Integer>

    <Port Name="MyPort"/>

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestSelector");

    NODE_POINTER( Integer, Entry )
    NODE_POINTER( Integer, Selector )

    // create and initialize a test port
    CTestPort Port;
    for (int i=40; i<50; ++i)
        Port.CreateEntry(i, i-40UL, RW);

    // connect the node map to the port
    Camera._Connect(&Port, "MyPort");

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)21, NumBagEntries);

    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    // print the string's content to stdout (don't forget to reset the filte pointer to the begin)
    cout << File.rdbuf();
    File.seekg( ios::beg );

    // load string into another(!) bag
    CFeatureBag FeatureBag2;
    File >> FeatureBag2;

    // change the values
    for (int64_t i=ptrSelector->GetMin(); i<=ptrSelector->GetMax(); i+=ptrSelector->GetInc())
        ptrEntry->SetValue(-1);
    *ptrSelector = 42;

    // load the values from FeatureBag and verify
    FeatureBag2.LoadFromBag(Camera._Ptr);
    for (int64_t i=ptrSelector->GetMin(); i<=ptrSelector->GetMax(); i+=ptrSelector->GetInc())
    {
        *ptrSelector = i;
        CPPUNIT_ASSERT_EQUAL( (int64_t)(i-40), ptrEntry->GetValue() );
    }

}

void NodeMapPersistenceTestSuite::TestAccessMode()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestAccessMode.xml|gxml

    <Category Name="Root">
        <pFeature>IntegerValueWasNI</pFeature>
    </Category>

    <Integer Name="IntegerValueWasNI">
        <pIsImplemented>Implemented</pIsImplemented>
        <Streamable>Yes</Streamable>
        <Value>17</Value>
    </Integer>

    <Integer Name="Implemented">
        <Value>0</Value>
    </Integer>

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestAccessMode");

    NODE_POINTER( Integer, IntegerValueWasNI )
    NODE_POINTER( Integer, Implemented )

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)0, NumBagEntries);


    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    // print the string's content to stdout (don't forget to reset the filte pointer to the begin)
    cout << File.rdbuf();
    File.seekg( ios::beg );

    // load string into another(!) bag
    CFeatureBag FeatureBag2;
    File >> FeatureBag2;

    // now make the before-NI node implemented, its value should not be altered by the persistence
    *ptrImplemented = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t)17, ptrIntegerValueWasNI->GetValue() );
    ptrIntegerValueWasNI->SetValue (23);

    // load the values from FeatureBag and verify
    FeatureBag2.LoadFromBag(Camera._Ptr);
    CPPUNIT_ASSERT_EQUAL( (int64_t)23, ptrIntegerValueWasNI->GetValue() );

}

 
void NodeMapPersistenceTestSuite::TestMisc()
{

    // EatComments should work also if at eof
    stringstream File;
    CPPUNIT_ASSERT_NO_THROW (EatComments(File));
    CPPUNIT_ASSERT_NO_THROW (EatComments(File));

    // On the other hand, FeatureBag's stream input should fail
    CFeatureBag FeatureBag;
    CPPUNIT_ASSERT_THROW_EX (File >> FeatureBag, GenICam::RuntimeException);

    // try to use a node map without "Root"
    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestMiscC.xml|gxml

    <Integer Name="IntegerValue1">
        <Streamable>Yes</Streamable>
        <Value>17</Value>
    </Integer>

    */
    CNodeMapRef CameraC;
    CameraC._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestMiscC");

    CFeatureBag FeatureBagC;
    FeatureBag.StoreToBag(CameraC._Ptr);


    // stream content of feature bag to string
    stringstream FileC;
    FileC << FeatureBag;

    // Check if nothing was streamed
    gcstring csRead( FileC.str().c_str() );
    CPPUNIT_ASSERT (csRead.find("IntegerValue1") == gcstring::npos);

    // now send the (empty) bag to a stream, skip the magic line, and observe the exception
    stringstream File2;
    File2 << FeatureBag;
    File2.seekg( ios::beg );
    File2.ignore(1024, '\n');
    CFeatureBag FeatureBag2;
    CPPUNIT_ASSERT_THROW_EX (File2 >> FeatureBag2, GenICam::RuntimeException);

    // try to load from an invalid pointer
    CFeatureBag FeatureBag4;
    CPPUNIT_ASSERT_THROW_EX (FeatureBag4.LoadFromBag(NULL), GenICam::InvalidArgumentException);


    // if a node name is not found, the LoadFromBag() should fail

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestMiscA.xml|gxml

    <Category Name="Root">
        <pFeature>IntegerValue1</pFeature>
    </Category>

    <Integer Name="IntegerValue1">
        <Streamable>Yes</Streamable>
        <Value>17</Value>
    </Integer>

    */

    CNodeMapRef CameraA;
    CameraA._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestMiscA");

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestMiscB.xml|gxml

    <Category Name="Root">
        <pFeature>IntegerValue2</pFeature>
    </Category>

    <Integer Name="IntegerValue2">
        <Streamable>Yes</Streamable>
        <Value>17</Value>
    </Integer>

    */

    CNodeMapRef CameraB;
    CameraB._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestMiscB");

    CFeatureBag FeatureBagA;
    FeatureBagA.StoreToBag(CameraA._Ptr);
    gcstring_vector ErrorList;
    FeatureBagA.LoadFromBag(CameraB._Ptr, true, &ErrorList);
    gcstring_vector::iterator pError;
    for( pError = ErrorList.begin(); pError != ErrorList.end(); pError++ )
    {
        cout << (*pError).c_str() << "\n";
    }
    CPPUNIT_ASSERT_EQUAL( (size_t)1, ErrorList.size() );

}

void NodeMapPersistenceTestSuite::TestEnumerationSelector()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestEnumerationSelector.xml|gxml
    
    <Category Name="Root">
        <pFeature>Entry</pFeature>
        <pFeature>Selector</pFeature>
    </Category>

    <Integer Name="Entry">
        <Streamable>Yes</Streamable>
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
        <Representation>Linear</Representation>
    </Integer>

    <Enumeration Name="Selector">
        <EnumEntry Name="Entry0" NameSpace="Standard">
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Entry1" NameSpace="Standard">
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="Entry2" NameSpace="Standard">
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>Entry</pSelected>
    </Enumeration>
    
    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestEnumerationSelector");

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)7, NumBagEntries);


    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    gcstring csRead( File.str().c_str() );

    // Check if all selector values were streamed out
    CPPUNIT_ASSERT (csRead.find("Entry0") != gcstring::npos);
    CPPUNIT_ASSERT (csRead.find("Entry1") != gcstring::npos);
    CPPUNIT_ASSERT (csRead.find("Entry2") != gcstring::npos);
    // Check value that restores original selector setting
    CPPUNIT_ASSERT (csRead.find("Entry1", csRead.find("Entry1") + 1) != gcstring::npos);
}


void NodeMapPersistenceTestSuite::TestEnumerationSelectorSelector()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestEnumerationSelectorSelector.xml|gxml
    
    <Category Name="Root">
        <pFeature>OuterSelector</pFeature>
        <pFeature>InnerSelector</pFeature>
        <pFeature>Feature</pFeature>
    </Category>

    <Integer Name="Feature">
        <Streamable>Yes</Streamable>
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
        <Representation>Linear</Representation>
    </Integer>

    <Enumeration Name="InnerSelector">
        <EnumEntry Name="InnerEntry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="InnerEntry1" >
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="InnerEntry2" >
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>Feature</pSelected>
    </Enumeration>

    <Enumeration Name="OuterSelector">
        <EnumEntry Name="OuterEntry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="OuterEntry1" >
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="OuterEntry2" >
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>InnerSelector</pSelected>
    </Enumeration>
    
    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestEnumerationSelectorSelector");

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)23, NumBagEntries);


    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    gcstring csRead( File.str().c_str() );
    cout << csRead << endl;

    // Check if the script is correct.
    size_t OuterPos[ 3 ];
    OuterPos[0] = csRead.find("OuterEntry0");
    OuterPos[1] = csRead.find("OuterEntry1");
    OuterPos[2] = csRead.find("OuterEntry2");
    for ( int i = 0; i < 3; ++i )
        CPPUNIT_ASSERT( OuterPos[i] != gcstring::npos );

    // For each value of OuterSelector we expect 3 values of InnerSelector.
    for ( int i = 0; i < 3; ++i )
    {
        size_t InnerPos[3];
        InnerPos[0] = csRead.find("InnerEntry0", OuterPos[i] );
        InnerPos[1] = csRead.find("InnerEntry1", OuterPos[i] );
        InnerPos[2] = csRead.find("InnerEntry2", OuterPos[i] );
        for ( int j = 0; j < 3; ++j )
            CPPUNIT_ASSERT( InnerPos[j] != gcstring::npos );
        // check if the InnerEntrys are located before the next OuterEntry
        if ( i < 2 )
        {
            for ( int j = 0; j< 3; ++j )
            {
                CPPUNIT_ASSERT( InnerPos[ j ] < OuterPos[i + 1] );
            }
        }
        // For each InnerSelector value we expect one value for the Feature node:
        for ( int j = 0; j < 3; ++j )
        {
            size_t entryPos = csRead.find( "Feature", InnerPos[j] );
            CPPUNIT_ASSERT( entryPos != gcstring::npos );
            if ( j < 2 )
                CPPUNIT_ASSERT( entryPos < InnerPos[ j + 1] );  // feature must be found in between InnerEntry[j] and InnerEntry[j+1]
        }
    }

    // Check values that restore original selector setting
    CPPUNIT_ASSERT (csRead.find("OuterEntry1", csRead.find("OuterEntry1") + 1) != gcstring::npos);
    CPPUNIT_ASSERT (csRead.find("InnerEntry1", csRead.find("InnerEntry1") + 1) != gcstring::npos);

    //load stored features to check whether the selectors are set back to their original setting
    CFeatureBag FeatureBag2;
    File >> FeatureBag2;

    gcstring innerSelector = CValuePtr( Camera._GetNode("InnerSelector"))->ToString();
    gcstring outerSelector = CValuePtr( Camera._GetNode("OuterSelector"))->ToString();

    FeatureBag2.LoadFromBag(Camera._Ptr);

    CPPUNIT_ASSERT_EQUAL( innerSelector, CValuePtr( Camera._GetNode("InnerSelector"))->ToString());
    CPPUNIT_ASSERT_EQUAL( outerSelector, CValuePtr( Camera._GetNode("OuterSelector"))->ToString());
}

 
void NodeMapPersistenceTestSuite::TestRecursionBreaker()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestRecursionBreaker.xml|gxml
    
    <Category Name="Root">
        <pFeature>OuterSelector</pFeature>
        <pFeature>InnerSelector</pFeature>
        <pFeature>Feature</pFeature>
    </Category>

    <Integer Name="Feature">
        <Streamable>Yes</Streamable>
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
        <Representation>Linear</Representation>
        <!-- Recursion! -->
        <pSelected>OuterSelector</pSelected>
    </Integer>

    <Enumeration Name="InnerSelector">
        <EnumEntry Name="InnerEntry0" >
            <Value>0</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>Feature</pSelected>
    </Enumeration>

    <Enumeration Name="OuterSelector">
        <EnumEntry Name="OuterEntry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="OuterEntry1" >
            <Value>1</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>InnerSelector</pSelected>
    </Enumeration>
       
    */


    CNodeMapRef Camera;
    CPPUNIT_ASSERT_THROW_EX( Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestRecursionBreaker"), GenICam::RuntimeException );

}


// Test for Trac #677
// A selector that is not implemented selects a feature that is implemented and streamable.
// The expectation is that the feature will be serialized.
void NodeMapPersistenceTestSuite::TestSelectorNotImplemented_1()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestSelectorNotImplemented_1.xml|gxml
    
    <Category Name="Root">
        <pFeature>Selector</pFeature>
        <pFeature>Selectee</pFeature>
    </Category>

    <Integer Name="Selectee">
        <Streamable>Yes</Streamable>
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
    </Integer>

    <Enumeration Name="Selector">
        <pIsImplemented>False</pIsImplemented>
        <EnumEntry Name="Entry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Entry1" >
            <Value>1</Value>
        </EnumEntry>
        <Value>0</Value>
        <pSelected>Selectee</pSelected>
    </Enumeration>

    <Integer Name="False">
        <Value>0</Value>
    </Integer>
    
    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestSelectorNotImplemented_1");

    CPPUNIT_ASSERT( ! IsImplemented( Camera._GetNode("Selector")) );

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)1, NumBagEntries);

    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    gcstring csRead( File.str().c_str() );
    cout << csRead << endl;

    // Check if the script is correct.
    // We expect that Selectee has ben written to the script although the selector is not implemented
    CPPUNIT_ASSERT( csRead.find("Selectee") != gcstring::npos );

}

// Test for Trac #677
// Two selectors select a feature. Both selectors are not implemented.
// The expectation is that the feature will be serialized.
void NodeMapPersistenceTestSuite::TestSelectorNotImplemented_2()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestSelectorNotImplemented_2.xml|gxml
    
    <Category Name="Root">
        <pFeature>Selector1</pFeature>
        <pFeature>Selector2</pFeature>
        <pFeature>Selectee</pFeature>
    </Category>

    <Integer Name="Selectee">
        <Streamable>Yes</Streamable>
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
    </Integer>

    <Enumeration Name="Selector1">
        <pIsImplemented>False</pIsImplemented>
        <EnumEntry Name="Entry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Entry1" >
            <Value>1</Value>
        </EnumEntry>
        <Value>0</Value>
        <pSelected>Selectee</pSelected>
    </Enumeration>

    <Enumeration Name="Selector2">
        <pIsImplemented>False</pIsImplemented>
        <EnumEntry Name="Entry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Entry1" >
            <Value>1</Value>
        </EnumEntry>
        <Value>0</Value>
        <pSelected>Selectee</pSelected>
    </Enumeration>

    <Integer Name="False">
        <Value>0</Value>
    </Integer>

    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestSelectorNotImplemented_2");

    CPPUNIT_ASSERT( ! IsImplemented( Camera._GetNode("Selector1")));
    CPPUNIT_ASSERT( ! IsImplemented( Camera._GetNode("Selector2")));

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)1, NumBagEntries);


    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    gcstring csRead( File.str().c_str() );
    cout << csRead << endl;

    // Check if the script is correct.
    // We expect that Selectee has ben written to the script although the selector is not implemented
    CPPUNIT_ASSERT( csRead.find("Selectee") != gcstring::npos );
}

// Test for Trac #677
// Two selectors select a feature. One selector is not implemented.
// The expectation is that the feature will be serialized.
void NodeMapPersistenceTestSuite::TestSelectorNotImplemented_3()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestSelectorNotImplemented_3.xml|gxml

    <Category Name="Root">
        <pFeature>Selector1</pFeature>
        <pFeature>Selector2</pFeature>
        <pFeature>Selectee</pFeature>
    </Category>

    <Integer Name="Selectee">
        <Streamable>Yes</Streamable>
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
        <Representation>Linear</Representation>
    </Integer>

    <Enumeration Name="Selector1">
        <pIsImplemented>False</pIsImplemented>
        <EnumEntry Name="S1Entry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="S1Entry1" >
            <Value>1</Value>
        </EnumEntry>
        <Value>0</Value>
        <pSelected>Selectee</pSelected>
    </Enumeration>

    <Enumeration Name="Selector2">
        <EnumEntry Name="S2Entry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="S2Entry1" >
            <Value>1</Value>
        </EnumEntry>
        <Value>0</Value>
        <pSelected>Selectee</pSelected>
    </Enumeration>
    
    <Integer Name="False">
        <Value>0</Value>
        <Min>0</Min>
        <Max>0</Max>
        <Representation>Linear</Representation>
    </Integer>

    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestSelectorNotImplemented_3");

    CPPUNIT_ASSERT( ! IsImplemented( Camera._GetNode("Selector1")));
    CPPUNIT_ASSERT(   IsImplemented( Camera._GetNode("Selector2")));

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)5, NumBagEntries);

    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    gcstring csRead( File.str().c_str() );
    cout << csRead << endl;

    // Check if the script is correct.
    // We expect that InnerSelector has been iterated

    size_t Pos[2];
    Pos[0] = csRead.find("S2Entry0");
    Pos[1] = csRead.find("S2Entry1");
    for ( int j = 0; j < 2; ++j )
        CPPUNIT_ASSERT( Pos[j] != gcstring::npos );
    // For each InnerSelector value we expect one value for the Feature node:
    for ( int j = 0; j < 2; ++j )
    {
        size_t entryPos = csRead.find( "Selectee", Pos[j] );
        CPPUNIT_ASSERT( entryPos != gcstring::npos );
        if ( j < 1 )
            CPPUNIT_ASSERT( entryPos < Pos[ j + 1] );  // feature must be found in between Entry[j] and Entry[j+1]
    }

    // Check value that restores original selector setting
    CPPUNIT_ASSERT (csRead.find("S2Entry0", Pos[0] + 1) != gcstring::npos);
}

// Test for Trac #677
// A selector that is not implemented selects two features that are implemented and streamable.
// The expectation is that the features will be serialized.
void NodeMapPersistenceTestSuite::TestSelectorNotImplemented_4()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestSelectorNotImplemented_4.xml|gxml

    <Category Name="Root">
        <pFeature>Selector</pFeature>
        <pFeature>Selectee1</pFeature>
        <pFeature>Selectee2</pFeature>
    </Category>

    <Integer Name="Selectee1">
        <Streamable>Yes</Streamable>
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
        <Representation>Linear</Representation>
    </Integer>

    <Integer Name="Selectee2">
        <Streamable>Yes</Streamable>
        <Value>1023</Value>
        <Min>192</Min>
        <Max>1023</Max>
        <Representation>Linear</Representation>
    </Integer>

    <Enumeration Name="Selector">
        <pIsImplemented>False</pIsImplemented>
        <EnumEntry Name="Entry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Entry1" >
            <Value>1</Value>
        </EnumEntry>
        <Value>0</Value>
        <pSelected>Selectee1</pSelected>
        <pSelected>Selectee2</pSelected>
    </Enumeration>

    <Integer Name="False">
        <Value>0</Value>
        <Min>0</Min>
        <Max>0</Max>
        <Representation>Linear</Representation>
    </Integer>

    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestSelectorNotImplemented_4");

    CPPUNIT_ASSERT( ! GenApi::IsImplemented( Camera._GetNode("Selector")));

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)2, NumBagEntries);


    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    gcstring csRead( File.str().c_str() );
    cout << csRead << endl;

    // Check if the script is correct.
    // We expect that the Selectees have ben written to the script although the selector is not implemented
    CPPUNIT_ASSERT( csRead.find("Selectee1") != gcstring::npos );
    CPPUNIT_ASSERT( csRead.find("Selectee2") != gcstring::npos );

}

// Test for Trac #677
// An outer selector selects an inner selector that selects a feature that is implemented and streamable.
// The outer selector is not implemented.
// The expectation is that the inner selector is iterated and the feature will be serialized.
void NodeMapPersistenceTestSuite::TestSelectorNotImplemented_5()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestSelectorNotImplemented_5.xml|gxml
    
    <Category Name="Root">
        <pFeature>OuterSelector</pFeature>
        <pFeature>InnerSelector</pFeature>
        <pFeature>Feature</pFeature>
    </Category>

    <Integer Name="Feature">
        <Streamable>Yes</Streamable>
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
        <Representation>Linear</Representation>
    </Integer>

    <Enumeration Name="InnerSelector">
        <EnumEntry Name="InnerEntry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="InnerEntry1" >
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="InnerEntry2" >
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>Feature</pSelected>
    </Enumeration>

    <Enumeration Name="OuterSelector">
        <pIsImplemented>False</pIsImplemented>
        <EnumEntry Name="OuterEntry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="OuterEntry1" >
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="OuterEntry2" >
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>InnerSelector</pSelected>
    </Enumeration>

    <Integer Name="False">
        <Value>0</Value>
        <Min>0</Min>
        <Max>0</Max>
        <Representation>Linear</Representation>
    </Integer>

    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestSelectorNotImplemented_5");

    CPPUNIT_ASSERT( ! GenApi::IsImplemented( Camera._GetNode("OuterSelector")));
    CPPUNIT_ASSERT(  GenApi::IsImplemented( Camera._GetNode("InnerSelector")));

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)7, NumBagEntries);

    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    gcstring csRead( File.str().c_str() );
    cout << csRead << endl;

    // Check if the script is correct.
    // We expect that OuterSelector has not been serialized
    CPPUNIT_ASSERT_EQUAL( gcstring::npos, csRead.find("OuterEntry"));
    // We expect that InnerSelector has been iterated

    size_t InnerPos[3];
    InnerPos[0] = csRead.find("InnerEntry0");
    InnerPos[1] = csRead.find("InnerEntry1");
    InnerPos[2] = csRead.find("InnerEntry2");
    for ( int j = 0; j < 3; ++j )
        CPPUNIT_ASSERT( InnerPos[j] != gcstring::npos );
    // For each InnerSelector value we expect one value for the Feature node:
    for ( int j = 0; j < 3; ++j )
    {
        size_t entryPos = csRead.find( "Feature", InnerPos[j] );
        CPPUNIT_ASSERT( entryPos != gcstring::npos );
        if ( j < 2 )
            CPPUNIT_ASSERT( entryPos < InnerPos[ j + 1] );  // feature must be found in between InnerEntry[j] and InnerEntry[j+1]
    }

    // Check value that restores original selector setting
    CPPUNIT_ASSERT (csRead.find("InnerEntry1", InnerPos[1] + 1) != gcstring::npos);


}

// Test for Trac #677
// An outer selector selects an inner selector that selects a feature that is implemented and streamable.
// The inner selector is not implemented.
// The expectation is that the outer selector is ignored (!)
void NodeMapPersistenceTestSuite::TestSelectorNotImplemented_6()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestSelectorNotImplemented_6.xml|gxml
    
    <Category Name="Root">
        <pFeature>OuterSelector</pFeature>
        <pFeature>InnerSelector</pFeature>
        <pFeature>Feature</pFeature>
    </Category>

    <Integer Name="Feature">
        <Streamable>Yes</Streamable>
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
        <Representation>Linear</Representation>
    </Integer>

    <Enumeration Name="InnerSelector">
        <pIsImplemented>False</pIsImplemented>
        <EnumEntry Name="InnerEntry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="InnerEntry1" >
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="InnerEntry2" >
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>Feature</pSelected>
    </Enumeration>

    <Enumeration Name="OuterSelector">
        <EnumEntry Name="OuterEntry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="OuterEntry1" >
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="OuterEntry2" >
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>InnerSelector</pSelected>
    </Enumeration>

    <Integer Name="False">
        <Value>0</Value>
        <Min>0</Min>
        <Max>0</Max>
        <Representation>Linear</Representation>
    </Integer>

    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestSelectorNotImplemented_6");

    CPPUNIT_ASSERT( GenApi::IsImplemented( Camera._GetNode("OuterSelector")));
    CPPUNIT_ASSERT( ! GenApi::IsImplemented( Camera._GetNode("InnerSelector")));

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)1, NumBagEntries);

    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    gcstring csRead( File.str().c_str() );
    cout << csRead << endl;

    // Check if the script is correct.
    // We expect that neither the inner nor the outer selector has not been serialized
    CPPUNIT_ASSERT_EQUAL( gcstring::npos, csRead.find("InnerEntry"));
    CPPUNIT_ASSERT_EQUAL( gcstring::npos, csRead.find("OuterEntry"));

}

// check that features becoming not implemented for some selector combination
// are not serialized
void NodeMapPersistenceTestSuite::TestSelectorNotImplemented_7()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestSelectorNotImplemented_7.xml|gxml
    
    <Category Name="Root">
        <pFeature>Selector</pFeature>
        <pFeature>Feature</pFeature>
    </Category>

    <Integer Name="Feature">
        <pIsImplemented>Selector</pIsImplemented>
        <Streamable>Yes</Streamable>
        <Value>42</Value>
    </Integer>

    <Enumeration Name="Selector">
        <EnumEntry Name="NotImplemented" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Implemented" >
            <Value>1</Value>
        </EnumEntry>
        <Value>0</Value>
        <pSelected>Feature</pSelected>
    </Enumeration>

    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestSelectorNotImplemented_7");

    NODE_POINTER( Integer, Feature )
    NODE_POINTER( Enumeration, Selector )

    *ptrSelector = "NotImplemented";
    CPPUNIT_ASSERT( !IsImplemented( ptrFeature ));

    *ptrSelector = "Implemented";
    CPPUNIT_ASSERT( IsImplemented( ptrFeature ));

    // persist the values to storage
    CFeatureBag FeatureBag;
    int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
    cout << "NumBagEntries = " << NumBagEntries << endl;
    CPPUNIT_ASSERT_EQUAL((int64_t)3, NumBagEntries);

    // stream content of feature bag to string
    stringstream File;
    File << FeatureBag;

    gcstring csRead( File.str().c_str() );
    cout << csRead << endl;

    // Check if the script is correct.
    // We expect that the feature is serialized only for Selector=Implemented 
    CPPUNIT_ASSERT( gcstring::npos != csRead.find("Selector\tImplemented"));
    CPPUNIT_ASSERT( gcstring::npos == csRead.find("Selector\tNotImplemented"));

    // Check value that restores original selector setting
    CPPUNIT_ASSERT (csRead.find("Selector\tImplemented", csRead.find("Selector\tImplemented") + 1) != gcstring::npos);
}

void NodeMapPersistenceTestSuite::TestMaxNumPersistSkriptEntries()
{

    /*[ GenApiTest@NodeMapPersistenceTestSuite_TestMaxNumPersistSkriptEntries.xml|gxml

    <Category Name="Root">
        <pFeature>IntegerValue</pFeature>
        <pFeature>IntegerValueNotStreamable</pFeature>
        <pFeature>FloatValue</pFeature>
        <pFeature>EnumerationValue</pFeature>
    </Category>

    <Integer Name="IntegerValue">
        <Streamable>Yes</Streamable>
        <Value>17</Value>
    </Integer>

    <Integer Name="IntegerValueNotStreamable">
        <Streamable>No</Streamable>
        <Value>17</Value>
    </Integer>

    <Float Name="FloatValue">
        <Streamable>Yes</Streamable>
        <Value>17.0</Value>
    </Float>

    <Enumeration Name="EnumerationValue">
        <Streamable>Yes</Streamable>
        <EnumEntry Name="EnumValue1">
            <Value>10</Value>
        </EnumEntry>
        <EnumEntry Name="EnumValue2">
            <Value>20</Value>
        </EnumEntry>
        <pValue>EnumInt</pValue>
    </Enumeration>

    <Integer Name="EnumInt">
        <Value>10</Value>
    </Integer>

    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "NodeMapPersistenceTestSuite_TestMaxNumPersistSkriptEntries");

    {
        // persist the values to storage
        CFeatureBag FeatureBag;
        int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr);
        cout << "NumBagEntries = " << NumBagEntries << endl;
        CPPUNIT_ASSERT_EQUAL((int64_t)3, NumBagEntries);

        // stream content of feature bag to string
        stringstream File;
        File << FeatureBag;

        // print the string's content to stdout (don't forget to reset the filte pointer to the begin)
        cout << File.rdbuf();
        File.seekg( ios::beg );
    }
    {
        // persist the values to storage with max 2 entries in the bag
        CFeatureBag FeatureBag;
        int64_t NumBagEntries = FeatureBag.StoreToBag(Camera._Ptr, 2);
        cout << "NumBagEntries = " << NumBagEntries << endl;
        CPPUNIT_ASSERT_EQUAL((int64_t)2, NumBagEntries);

        // stream content of feature bag to string
        stringstream File;
        File << FeatureBag;

        // print the string's content to stdout (don't forget to reset the filte pointer to the begin)
        cout << File.rdbuf();
        File.seekg( ios::beg );
    }


}
