//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "RegisterPortTest.h"
#include "TestRegisterPort.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( RegisterPortTestSuite, "GenApiTest");


void RegisterPortTestSuite::TestTestRegisterPort()
{
    // Test IPort::Read quadlet aligned
    {
        CTestRegisterPort Port(2);

        //write something to the register map
        uint32_t MyRegister = 42;
        Port.CreateEntry(0x0100, sizeof(MyRegister), &MyRegister, RO);

        // check if it's there
        uint32_t Buffer = 0;
        CPortPtr ptrPort = &Port;
        ptrPort->Read(&Buffer, 0x0100, sizeof(Buffer));
        CPPUNIT_ASSERT_EQUAL(MyRegister, Buffer);

        // try to read from an invalid address
        CPPUNIT_ASSERT_THROW_EX(
            ptrPort->Read(&Buffer, 0x0000 /*invalid*/, sizeof(Buffer)),
            GenICam::RuntimeException
        );

        // try to read with invalid length
        CPPUNIT_ASSERT_THROW_EX(
            ptrPort->Read(&Buffer, 0x0100, 8 /*invalid*/),
            GenICam::RuntimeException
        );

        // Create an WO entry
        MyRegister = 43;
        Port.CreateEntry(0x0010, sizeof(MyRegister), &MyRegister, WO);

        // try to read from a WO address
        CPPUNIT_ASSERT_THROW_EX(
            ptrPort->Read(&Buffer, 0x0010, sizeof(Buffer)),
            GenICam::RuntimeException
        );

    }

    // Test IPort::Read not quadlet aligned
    {
        CTestRegisterPort Port(16);

        //write something to the register map
        uint8_t pRegisterBuffer[8] = {0};
        uint32_t MyRegister = 42;
        memcpy(pRegisterBuffer + 3, (void*)&MyRegister, 4);

        Port.CreateEntry(0x0000, 8, pRegisterBuffer, RO);

        // check if it's there
        uint32_t Buffer = 0;
        CPortPtr ptrPort = &Port;
        ptrPort->Read(&Buffer, 0x0003, 4);
        CPPUNIT_ASSERT_EQUAL(MyRegister, Buffer);
    }

    // Test IPort::Read not quadlet aligned & maxQuadlets 1
    {
        CTestRegisterPort Port(1);

        //write something to the register map
        uint8_t pRegisterBuffer[8] = {0};
        uint32_t MyRegister = 42;
        memcpy(pRegisterBuffer + 3, (void*)&MyRegister, 4);

        Port.CreateEntry(0x0000, 4, pRegisterBuffer, RO);
        Port.CreateEntry(0x0004, 4, pRegisterBuffer+4, RO);

        // check if it's there
        uint32_t Buffer = 0;
        CPortPtr ptrPort = &Port;
        ptrPort->Read(&Buffer, 0x0003, 4);
        CPPUNIT_ASSERT_EQUAL(MyRegister, Buffer);
    }


    // Test IPort::Write quadlet aligned
    {
        CTestRegisterPort Port(2);

        //write something to the register map
        uint32_t MyRegister = 0;
        Port.CreateEntry(0x0100, sizeof(MyRegister), &MyRegister, RW);

        // write at the location
        MyRegister = 42;
        CPortPtr ptrPort = &Port;
        ptrPort->Write(&MyRegister, 0x0100, sizeof(MyRegister));

        // read back
        uint32_t Buffer = 0;
        ptrPort->Read(&Buffer, 0x0100, sizeof(Buffer));
        CPPUNIT_ASSERT_EQUAL(MyRegister, Buffer);

        // try to write at an invalid address
        CPPUNIT_ASSERT_THROW_EX(
            ptrPort->Write(&MyRegister, 0x0000 /*invalid*/, sizeof(MyRegister)),
            GenICam::RuntimeException
        );

        // try to write with invalid length
        CPPUNIT_ASSERT_THROW_EX(
            ptrPort->Write(&MyRegister, 0x0100, 8 /*invalid*/),
            GenICam::RuntimeException
        );

        //make the entry RO
        MyRegister = 0;
        Port.CreateEntry(0x0010, sizeof(MyRegister), &MyRegister, RO);

        // try to write to a RO address
        CPPUNIT_ASSERT_THROW_EX(
            ptrPort->Write(&MyRegister, 0x0010, sizeof(MyRegister)),
            GenICam::RuntimeException
        );

    }

    // Test IPort::Write not quadlet aligned
    {
        CTestRegisterPort Port(16);

        //create space in the register map
        uint8_t pRegisterBuffer[8] = {0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF};
        Port.CreateEntry(0x0000, 8, pRegisterBuffer, RW);
        Port.CreateEntry(0x0004, 4, pRegisterBuffer+4, RW);

        uint32_t MyRegister = 42;
        CPortPtr ptrPort = &Port;
        ptrPort->Write(&MyRegister, 0x0003, 4);

        // check if it's there
        uint32_t Buffer = 0;
        ptrPort->Read(&Buffer, 0x0003, 4);
        CPPUNIT_ASSERT_EQUAL(MyRegister, Buffer);

        uint8_t pTestBuffer[8] = {0};
        uint8_t pCompareBuffer[8] = {0x01, 0x23, 0x45, 0x2A, 0x00, 0x00, 0x00, 0xEF};
        EAccessMode TestAccessMode;
        Port.LookupEntry(0x0000, 8, pTestBuffer, TestAccessMode);
        for(int i=0; i<8; i++)
            CPPUNIT_ASSERT_EQUAL(pTestBuffer[i], pCompareBuffer[i]);
    }

    // Test IPort::Write not quadlet aligned & maxquadlets = 1
    {
        CTestRegisterPort Port(1);

        //create space in the register map
        uint8_t pRegisterBuffer[8] = {0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF};
        Port.CreateEntry(0x0000, 4, pRegisterBuffer, RW);
        Port.CreateEntry(0x0004, 4, pRegisterBuffer+4, RW);

        uint32_t MyRegister = 42;
        CPortPtr ptrPort = &Port;
        ptrPort->Write(&MyRegister, 0x0003, 4);

        // check if it's there
        uint32_t Buffer = 0;
        ptrPort->Read(&Buffer, 0x0003, 4);
        CPPUNIT_ASSERT_EQUAL(MyRegister, Buffer);

        uint8_t pTestBuffer[8] = {0};
        uint8_t pCompareBuffer[8] = {0x01, 0x23, 0x45, 0x2A, 0x00, 0x00, 0x00, 0xEF};
        EAccessMode TestAccessMode;
        Port.LookupEntry(0x0000, 4, pTestBuffer, TestAccessMode);
        Port.LookupEntry(0x0004, 4, pTestBuffer+4, TestAccessMode);
        for(int i=0; i<8; i++)
            CPPUNIT_ASSERT_EQUAL(pTestBuffer[i], pCompareBuffer[i]);
    }

    // Test IPort::Write not quadlet aligned & 1 byte only
    {
        CTestRegisterPort Port(16);

        //create space in the register map
        uint8_t pRegisterBuffer[4] = {0x01, 0x23, 0x45, 0x67 };
        Port.CreateEntry(0x0000, 4, pRegisterBuffer, RW);

        uint8_t MyRegister = 42;
        CPortPtr ptrPort = &Port;
        ptrPort->Write(&MyRegister, 0x0002, 1);

        // check if it's there
        uint8_t Buffer = 0;
        ptrPort->Read(&Buffer, 0x0002, 1);
        CPPUNIT_ASSERT_EQUAL(MyRegister, Buffer);

        uint8_t pTestBuffer[4] = {0};
        uint8_t pCompareBuffer[8] = {0x01, 0x23, 0x2A, 0x67};
        EAccessMode TestAccessMode;
        Port.LookupEntry(0x0000, 4, pTestBuffer, TestAccessMode);
        for(int i=0; i<4; i++)
            CPPUNIT_ASSERT_EQUAL(pTestBuffer[i], pCompareBuffer[i]);
    }
}
void RegisterPortTestSuite::TestPortAccess()
{

    // No swap I
    {
        // create a camera description file
        gcstring CameraDescriptionFile =

        CDFHeader +
        "<IntReg Name=\"Value\">                \n"
        "    <Address>0x0100</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Sign>Unsigned</Sign>                    \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"
        "                                        \n"
        "    <Port Name=\"Port\" >                \n"
        "    </Port>                                \n"
        + CDFFooter;

        // create and initialize a test port
        CTestRegisterPort Port(1, false);

        uint8_t pRegisterBuffer[4] = {0x01, 0x23, 0x45, 0x67 };
        Port.CreateEntry(0x0100, 4, pRegisterBuffer, RW);

        // create and connect a camera object
        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);
        Camera._Connect(&Port, "Port");

        CPortPtr ptrPort = Camera._GetNode("Port");
        CPPUNIT_ASSERT( ptrPort.IsValid() );

        CIntegerPtr ptrValue = Camera._GetNode("Value");
        CPPUNIT_ASSERT( ptrValue.IsValid() );

        CPPUNIT_ASSERT_EQUAL((int64_t)0x01234567, *ptrValue());

        *ptrValue = 0x89ABCDEF;

        CPPUNIT_ASSERT_EQUAL((int64_t)0x89ABCDEF, *ptrValue());

        uint8_t pTestBuffer[4] = {0};
        uint8_t pCompareBuffer[8] = {0x89, 0xAB, 0xCD, 0xEF};
        EAccessMode TestAccessMode;
        Port.LookupEntry(0x0100, 4, pTestBuffer, TestAccessMode);
        for(int i=0; i<4; i++)
            CPPUNIT_ASSERT_EQUAL(pTestBuffer[i], pCompareBuffer[i]);

    }

    // No swap II
    {
        // create a camera description file
        gcstring CameraDescriptionFile =

        CDFHeader +
        "<IntReg Name=\"Value\">                \n"
        "    <Address>0x0100</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Sign>Unsigned</Sign>                    \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"
        "                                        \n"
        "    <Port Name=\"Port\" >                \n"
        "        <SwapEndianess>Yes</SwapEndianess>    \n"
        "    </Port>                                \n"
        + CDFFooter;

        // create and initialize a test port
        CTestRegisterPort Port(1, true);

        uint8_t pRegisterBuffer[4] = {0x01, 0x23, 0x45, 0x67 };
        Port.CreateEntry(0x0100, 4, pRegisterBuffer, RW);

        // create and connect a camera object
        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);
        Camera._Connect(&Port, "Port");

        CPortPtr ptrPort = Camera._GetNode("Port");
        CPPUNIT_ASSERT( ptrPort.IsValid() );

        CIntegerPtr ptrValue = Camera._GetNode("Value");
        CPPUNIT_ASSERT( ptrValue.IsValid() );

        CPPUNIT_ASSERT_EQUAL((int64_t)0x01234567, *ptrValue());

        *ptrValue = 0x89ABCDEF;

        CPPUNIT_ASSERT_EQUAL((int64_t)0x89ABCDEF, *ptrValue());

        uint8_t pTestBuffer[4] = {0};
        uint8_t pCompareBuffer[8] = {0x89, 0xAB, 0xCD, 0xEF};
        EAccessMode TestAccessMode;
        Port.LookupEntry(0x0100, 4, pTestBuffer, TestAccessMode);
        for(int i=0; i<4; i++)
            CPPUNIT_ASSERT_EQUAL(pTestBuffer[i], pCompareBuffer[i]);

    }

    // swap I
    {
        // create a camera description file
        gcstring CameraDescriptionFile =

        CDFHeader +
        "<IntReg Name=\"Value\">                \n"
        "    <Address>0x0100</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Sign>Unsigned</Sign>                    \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"
        "                                        \n"
        "    <Port Name=\"Port\" >                \n"
        "    </Port>                                \n"
        + CDFFooter;

        // create and initialize a test port
        CTestRegisterPort Port(1, true);
        Port.InvalidateNode (); // this should now be no-op, fulfilling happy path

        uint8_t pRegisterBuffer[4] = {0x67, 0x45, 0x23, 0x01 };
        Port.CreateEntry(0x0100, 4, pRegisterBuffer, RW);

        // create and connect a camera object
        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);
        Camera._Connect(&Port, "Port");

        CPortPtr ptrPort = Camera._GetNode("Port");
        CPPUNIT_ASSERT( ptrPort.IsValid() );

        CIntegerPtr ptrValue = Camera._GetNode("Value");
        CPPUNIT_ASSERT( ptrValue.IsValid() );

        CPPUNIT_ASSERT_EQUAL((int64_t)0x01234567, *ptrValue());

        *ptrValue = 0x89ABCDEF;

        CPPUNIT_ASSERT_EQUAL((int64_t)0x89ABCDEF, *ptrValue());

        uint8_t pTestBuffer[4] = {0};
        uint8_t pCompareBuffer[8] = {0xEF, 0xCD, 0xAB, 0x89};
        EAccessMode TestAccessMode;
        Port.LookupEntry(0x0100, 4, pTestBuffer, TestAccessMode);
        for(int i=0; i<4; i++)
            CPPUNIT_ASSERT_EQUAL(pTestBuffer[i], pCompareBuffer[i]);
    }

    // swap II
    {
        // create a camera description file
        gcstring CameraDescriptionFile =

        CDFHeader +
        "<IntReg Name=\"Value\">                \n"
        "    <Address>0x0100</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Sign>Unsigned</Sign>                    \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"
        "                                        \n"
        "    <Port Name=\"Port\" >                \n"
        "        <SwapEndianess>Yes</SwapEndianess>    \n"
        "    </Port>                                \n"
        + CDFFooter;

        // create and initialize a test port
        CTestRegisterPort Port(1, false);

        uint8_t pRegisterBuffer[4] = {0x67, 0x45, 0x23, 0x01 };
        Port.CreateEntry(0x0100, 4, pRegisterBuffer, RW);

        // create and connect a camera object
        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);
        Camera._Connect(&Port, "Port");

        CPortPtr ptrPort = Camera._GetNode("Port");
        CPPUNIT_ASSERT( ptrPort.IsValid() );

        CIntegerPtr ptrValue = Camera._GetNode("Value");
        CPPUNIT_ASSERT( ptrValue.IsValid() );

        CPPUNIT_ASSERT_EQUAL((int64_t)0x01234567, *ptrValue());

        *ptrValue = 0x89ABCDEF;

        CPPUNIT_ASSERT_EQUAL((int64_t)0x89ABCDEF, *ptrValue());

        uint8_t pTestBuffer[4] = {0};
        uint8_t pCompareBuffer[8] = {0xEF, 0xCD, 0xAB, 0x89};
        EAccessMode TestAccessMode;
        Port.LookupEntry(0x0100, 4, pTestBuffer, TestAccessMode);
        for(int i=0; i<4; i++)
            CPPUNIT_ASSERT_EQUAL(pTestBuffer[i], pCompareBuffer[i]);
    }
  // Test Nopde Invalidation
  {
        // create a camera description file
        gcstring CameraDescriptionFile =

        CDFHeader +
        "<IntReg Name=\"Value\">                \n"
        "    <Address>0x0100</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Sign>Unsigned</Sign>                    \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"
        "                                        \n"
        "    <Port Name=\"Port\" >                \n"
        "        <SwapEndianess>Yes</SwapEndianess>    \n"
        "    </Port>                                \n"
        + CDFFooter;

    // create and initialize a test port
        CTestRegisterPort Port(1, false);

        uint8_t pRegisterBuffer[4] = {0x67, 0x45, 0x23, 0x01 };
        Port.CreateEntry(0x0100, 4, pRegisterBuffer, RW);

        // create and connect a camera object
        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);
        Camera._Connect(&Port, "Port");

        CPortPtr ptrPort = Camera._GetNode("Port");
        CPPUNIT_ASSERT( ptrPort.IsValid() );

    CPPUNIT_ASSERT_NO_THROW( Port.InvalidateNode() );


  }


}


