//-----------------------------------------------------------------------------
//  (c) 2006 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
\brief    A high resolution stop watch
*/

#ifndef __STOPWATCH_H_
#define __STOPWATCH_H_

#define TEST_BEGIN(Runs) \
{ \
    double fRuns = (double)Runs; \
    CStopWatch StopWatch; \
    StopWatch.Start(); \
    for(int i=0; i<Runs; i++) \
    {

#define TEST_END(Description) \
    } \
    double ElapsedTime = 1000000.0 * StopWatch.Stop() / fRuns; \
    CLog::GetLogger("CppUnit.Performance").info( #Description " : t = %.2f us", ElapsedTime ); \
}

#define LOG_PERFORMANCE \
    CLog::GetLogger("CppUnit.Performance").info

#if defined (_WIN32)

namespace GenICam
{
    //! Measures time intervals with high resolution
    class CStopWatch
    {
    public:
        //! constructor
        CStopWatch(void)
        {
            m_StartTime.QuadPart = 0ULL;
            m_StopTime.QuadPart = 0ULL;
            m_ElapsedTime.QuadPart = 0ULL;
            m_StartStopOverhead.QuadPart = 0ULL;
            m_PauseOverhead.QuadPart = 0ULL;

            Start();
            Stop();
            m_StartStopOverhead = m_ElapsedTime;

            Start();
            PauseBegin();
            PauseEnd();
            Stop();
            m_PauseOverhead = m_ElapsedTime;
        }

          //! Start the stop watch
          void Start(void)
          {
              m_ElapsedTime.QuadPart = 0ULL;
              QueryPerformanceCounter(&m_StartTime);
          }

          //! pause the time measurement
          void PauseBegin(void)
          {
              QueryPerformanceCounter(&m_StopTime);

              m_ElapsedTime.QuadPart += m_StopTime.QuadPart - m_StartTime.QuadPart - m_PauseOverhead.QuadPart;
          }

          //! restart the time measurement
          void PauseEnd(void)
          {
              QueryPerformanceCounter(&m_StartTime);
          }

          //! Stops the stop watch
          /*! The elapsed time is returned. The stop watch may be started again
          */
          double Stop(void)
          {
              QueryPerformanceCounter(&m_StopTime);

              m_ElapsedTime.QuadPart += m_StopTime.QuadPart - m_StartTime.QuadPart - m_StartStopOverhead.QuadPart;

              return ElapsedTime();
          }

          //! Returns the elapsed time in seconds between start() and stop()
          double ElapsedTime(void)
          {
              LARGE_INTEGER timerFrequency;
              QueryPerformanceFrequency(&timerFrequency);

              const long double
                timeDifference((long double) m_ElapsedTime.QuadPart),
                ticksPerSecond((long double) timerFrequency.QuadPart);

              return (double)(timeDifference / ticksPerSecond);
          }

          //! Returns the elapsed time in seconds between start() and stop()
          static double SecondsPerTick(void)
          {
              LARGE_INTEGER timerFrequency;
              QueryPerformanceFrequency(&timerFrequency);

              const long double
                ticksPerSecond((long double) timerFrequency.QuadPart);

              return (double)(1.0 / ticksPerSecond);
          }

    protected:
        //! zero-point for time measurment
        LARGE_INTEGER m_StartTime;

        //! last time stamp
        LARGE_INTEGER m_StopTime;

        //! sum of the elapsed time
        LARGE_INTEGER m_ElapsedTime;

        //! overhead for one pair of start/stop calls
        LARGE_INTEGER m_StartStopOverhead;

        //! overhead for one pair of PauseBegin/PauseEnd calls
        LARGE_INTEGER m_PauseOverhead;
    };
}

#elif defined (HAVE_CONFIG_H) && defined(HAVE_SYS_TIME_H)

#include <sys/time.h>

namespace GenICam
{
    //! Measures time intervals with high resolution
    class CStopWatch
    {
    public:
        //! constructor
        CStopWatch(void) :
            m_StartTime(0),
            m_StopTime(0),
            m_ElapsedTime(0),
            m_StartStopOverhead(0),
            m_PauseOverhead(0)
        {

            Start();
            Stop();
            m_StartStopOverhead = m_ElapsedTime;

            Start();
            PauseBegin();
            PauseEnd();
            Stop();
            m_PauseOverhead = m_ElapsedTime;
        }

          //! Start the stop watch
          void Start(void)
          {
              m_ElapsedTime = 0;
              m_StartTime = SampleTime();
          }

          //! pause the time measurement
          void PauseBegin(void)
          {
              m_StopTime = SampleTime();
              m_ElapsedTime += m_StopTime - m_StartTime - m_PauseOverhead;
          }

          //! restart the time measurement
          void PauseEnd(void)
          {
              m_StartTime = SampleTime();
          }

          //! Stops the stop watch
          /*! The elapsed time is returned. The stop watch may be started again
          */
          double Stop(void)
          {
              m_StopTime = SampleTime();
              m_ElapsedTime += m_StopTime - m_StartTime - m_StartStopOverhead;
              return ElapsedTime();
          }

          //! Returns the elapsed time in seconds between start() and stop()
          double ElapsedTime(void)
          {
              return m_ElapsedTime;
          }

          //! Returns the elapsed time in seconds between start() and stop()
          static double SecondsPerTick(void)
          {
              // No easy way to get at the true resolution, so...
              return 0.0;
          }

    protected:

        //! Return the current time as long double
        static long double SampleTime(void)
        {
            struct timeval tv;

            gettimeofday(&tv, NULL);
            return
                static_cast<long double>(tv.tv_sec) +
                static_cast<long double>(tv.tv_usec) * 1e-6;
        }

        //! zero-point for time measurment
        long double m_StartTime;

        //! last time stamp
        long double m_StopTime;

        //! sum of the elapsed time
        long double m_ElapsedTime;

        //! overhead for one pair of start/stop calls
        long double m_StartStopOverhead;

        //! overhead for one pair of PauseBegin/PauseEnd calls
        long double m_PauseOverhead;
    };
}

#else
#   error No known support for measuring elapsed time
#endif


#endif // #ifndef __STOPWATCH_H_
